#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>

#ifdef _MSC_VER
#pragma comment (lib, "SDL2.lib")
#pragma comment (lib, "SDL2main.lib")
#endif

#include <SDL.h>
#include <SDL_audio.h>
#include <SDL_opengl.h>

typedef struct
{
	uint8_t r, g, b, a;
} srgb_t;

typedef struct
{
	srgb_t *sfb;
	int w, h;
	
	void *window;
	void *renderer;
	void *texture;
} framebuffer_t;

void sdl_graphics_init_full(framebuffer_t *fb)
{
	if (SDL_Init(SDL_INIT_TIMER | SDL_INIT_VIDEO | SDL_INIT_AUDIO))
		fprintf(stderr, "SDL_Init failed: %s\n", SDL_GetError());

	fb->window = SDL_CreateWindow ("bug_test", SDL_WINDOWPOS_UNDEFINED, SDL_WINDOWPOS_UNDEFINED, fb->w, fb->h, SDL_WINDOW_OPENGL | SDL_WINDOW_RESIZABLE);
	if (fb->window==NULL)
		fprintf(stderr, "SDL_CreateWindow failed: %s\n", SDL_GetError());

	fb->renderer = SDL_CreateRenderer(fb->window, -1, SDL_RENDERER_PRESENTVSYNC);
	if (fb->renderer==NULL)
		fprintf(stderr, "SDL_CreateRenderer failed: %s\n", SDL_GetError());

	fb->texture = SDL_CreateTexture(fb->renderer, SDL_PIXELFORMAT_ABGR8888, SDL_TEXTUREACCESS_STREAMING, fb->w, fb->h);
	if (fb->texture==NULL)
		fprintf(stderr, "SDL_CreateTexture failed: %s\n", SDL_GetError());

	fb->sfb = calloc(fb->w * fb->h, sizeof(srgb_t));
}

SDL_Rect make_sdl_rect(int x, int y, int w, int h)
{
	SDL_Rect rs;

	rs.x = x;
	rs.y = y;
	rs.w = w;
	rs.h = h;

	return rs;
}

void draw_stuff(framebuffer_t *fb)
{
	int mx, my, ix, iy, v;

	SDL_GetMouseState(&mx, &my);

	for (iy=0; iy < fb->h; iy++)
		for (ix=0; ix < fb->w; ix++)
		{
			v = 1024 - abs(mx-ix) - abs(my-iy);
			v *= rand() & 0xFF;
			v >>= 10;
			v = abs(v);

			fb->sfb[iy*fb->w+ix].r = v;
			fb->sfb[iy*fb->w+ix].g = v;
			fb->sfb[iy*fb->w+ix].b = v;
		}
}

void sdl_flip_fb(framebuffer_t *fb)
{
	void *pixels;
	int pitch;
	SDL_Rect rs;

	// Displays framebuffer to screen
	rs = make_sdl_rect(0, 0, fb->w, fb->h);
	SDL_LockTexture(fb->texture, &rs, &pixels, &pitch);
	memcpy(pixels, fb->sfb, fb->w*fb->h * sizeof(srgb_t));
	SDL_UnlockTexture(fb->texture);

	SDL_RenderClear(fb->renderer);
	SDL_RenderCopy(fb->renderer, fb->texture, NULL, NULL);
	SDL_RenderPresent(fb->renderer);

	memset (fb->sfb, 0, fb->w*fb->h*sizeof(srgb_t));
}

int main()
{
	framebuffer_t fb={0};
	SDL_Event event={0};
	int exit_flag=0;

	fb.w = 640;
	fb.h = 480;
	sdl_graphics_init_full(&fb);

	while (exit_flag==0)
	{
		while (SDL_PollEvent(&event))
		{
			// were I to handle resizing properly it work crash after resize in sdl_flip_fb() because 'pixels' would be bogus

			if (event.type==SDL_QUIT)
				exit_flag = 1;
		}

		// Draw things here
		draw_stuff(&fb);

		// Display fb.sfb onto the screen
		sdl_flip_fb(&fb);
	}
}
